//
//               INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Intel Corporation and may not be copied
//  or disclosed except in accordance with the terms of that agreement.
//        Copyright (c) 2015 Intel Corporation. All Rights Reserved.
//

#pragma once

#include "windows.h"
#include "winbase.h"

class Clock
{
public:
    Clock()
        : m_duration(0), 
        m_count(0)
    {
        QueryPerformanceFrequency(&m_freq);
    }

    bool Begin()
    {
        return QueryPerformanceCounter(&m_begin);
    }

    bool End()
    {
        if (!QueryPerformanceCounter(&m_end))
        {
            return false;
        }

        double time = (double)(m_end.QuadPart - m_begin.QuadPart) / (double)m_freq.QuadPart * 1000;
        m_duration += time;

        ++m_count;

        return true;
    }

    double TimeSinceLastCall()
    {
        if (!QueryPerformanceCounter(&m_end))
        {
            return 0xDEADBEEF;
        }

        double ret = ( (double)(m_end.QuadPart - m_begin.QuadPart) / (double)m_freq.QuadPart * 1000 );
        m_begin = m_end;

        return ret;
    }

    double Duration() const { return m_duration; }
    double Average()  const { return (double) (m_count / m_duration); };
    int Count() const { return m_count; }

protected:
    LARGE_INTEGER m_freq;
    LARGE_INTEGER m_begin;
    LARGE_INTEGER m_end;

    double m_duration; // ms
    int    m_count;
};